-----------------------------------------------------------
--|> CoreFrame
--: Creates and updates the addon frames and widgets
-----------------------------------------------------------
--|> Upvalue Globals
-----------------------------------------------------------
local _, NS = ...
-- local InCombatLockdown = InCombatLockdown
local unpack = unpack
local pairs = pairs
local Round = Round
local InCombatLockdown = InCombatLockdown

--> Core UI Table Init
NS.CoreUI = NS.CoreUI or {}
local CUI = NS.CoreUI

--|> LIBS
-----------------------------------------------------------
local SM = LibStub:GetLibrary("LibSharedMedia-3.0")

--|> String Constants
-----------------------------------------------------------
local sc = {}
local defaultBackdrop = {}
local function localInit()
    sc = {
        white8x8 = "Interface/BUTTONS/WHITE8X8",
        pinned = "|cFF" .. NS.ColorsLUT["Pinned"].hex .. "PINNED|r",
        unpinned = "|cFF" .. NS.ColorsLUT["Unpinned"].hex .. "UNPINNED|r",
        locked = "|cFF" .. NS.ColorsLUT["Locked"].hex .. "LOCKED|r",
        unlocked = "|cFF" .. NS.ColorsLUT["Unlocked"].hex .. "UNLOCKED|r"
    }

    defaultBackdrop = {
        bgFile = sc.white8x8,
        edgeFile = sc.white8x8,
        edgeSize = 1
    }
end

--|> Local Build Functions
-----------------------------------------------------------
--> Core Frame

local function BuildCoreFrame()
    --> CoreFrame
    CUI.CoreFrame = CUI.CoreFrame or CreateFrame("Frame", "weizPVP_CoreFrame", UIParent)
    CUI.CoreFrame:SetFrameStrata("MEDIUM")
    CUI.CoreFrame:SetClampedToScreen(true)
    CUI.CoreFrame:SetDontSavePosition(true)
    CUI.CoreFrame:SetHeight(NS.Options.Bars.RowHeight * 10)
    CUI.CoreFrame:SetWidth(256)
    CUI.CoreFrame:SetResizable(true)
    CUI.CoreFrame:SetMinResize(180, NS.Options.Bars.RowHeight)
    CUI.CoreFrame:SetMaxResize(500, NS.Options.Frames.List.Height)
end

--> Build Header
local function BuildHeaderFrame()
    --> Header
    CUI.CoreFrame.Header = CUI.CoreFrame.Header or CreateFrame("Frame", nil, CUI.CoreFrame)
    CUI.CoreFrame.Header:SetPoint("BOTTOMLEFT", CUI.CoreFrame, "TOPLEFT", 0, -1)
    CUI.CoreFrame.Header:SetPoint("BOTTOMRIGHT", CUI.CoreFrame, "TOPRIGHT", 0, -1)
    CUI.CoreFrame.Header:SetClampedToScreen(true)
    CUI.CoreFrame.Header:SetHeight(NS.Options.Frames.Header.Height)
    CUI.CoreFrame.Header:SetWidth(256)
    CUI.CoreFrame.Header:EnableMouse(true)
    CUI.CoreFrame.Header:SetMovable(true)
    CUI.CoreFrame.Header:RegisterForDrag("LeftButton")
    CUI.CoreFrame.Header:SetResizable(true)
    CUI.CoreFrame.Header:SetMinResize(180, NS.Options.Frames.Header.Height)
    CUI.CoreFrame.Header:SetMaxResize(500, NS.Options.Frames.Header.Height)
    CUI.CoreFrame.Header:SetHitRectInsets(0, 40, 0, 0)
    CUI.CoreFrame.Header:SetBackdrop(defaultBackdrop)
    CUI.CoreFrame.Header:SetBackdropColor(unpack(NS.Options.Frames.Header.BackgroundColor))
    CUI.CoreFrame.Header:SetBackdropBorderColor(0, 0, 0, 1)

    --> Addon Logo
    CUI.CoreFrame.Header.AddonLogo = CUI.CoreFrame.Header.AddonLogo or CreateFrame("Frame", nil, CUI.CoreFrame.Header)
    CUI.CoreFrame.Header.AddonLogo:SetPoint("BOTTOMLEFT", CUI.CoreFrame.Header, "BOTTOMLEFT", 4, 0)
    CUI.CoreFrame.Header.AddonLogo:SetSize(NS.Options.Frames.Header.Height, NS.Options.Frames.Header.Height)
    --:bg
    CUI.CoreFrame.Header.AddonLogo.bg = CUI.CoreFrame.Header.AddonLogo.bg or CUI.CoreFrame.Header.AddonLogo:CreateTexture(nil, "ARTWORK")
    CUI.CoreFrame.Header.AddonLogo.bg:SetPoint("TOPLEFT", CUI.CoreFrame.Header.AddonLogo, "TOPLEFT")
    CUI.CoreFrame.Header.AddonLogo.bg:SetPoint("BOTTOMRIGHT", CUI.CoreFrame.Header.AddonLogo, "BOTTOMRIGHT")
    CUI.CoreFrame.Header.AddonLogo.bg:SetTexture("Interface/Addons/weizPVP/Media/weizpvp_nobg.tga")
    --: highlight
    CUI.CoreFrame.Header.AddonLogo.highlight =
        CUI.CoreFrame.Header.AddonLogo.highlight or CUI.CoreFrame.Header.AddonLogo:CreateTexture(nil, "ARTWORK")
    CUI.CoreFrame.Header.AddonLogo.highlight:SetDrawLayer("ARTWORK", 1)
    CUI.CoreFrame.Header.AddonLogo.highlight:SetPoint("TOPLEFT", CUI.CoreFrame.Header.AddonLogo, "TOPLEFT")
    CUI.CoreFrame.Header.AddonLogo.highlight:SetPoint("BOTTOMRIGHT", CUI.CoreFrame.Header.AddonLogo, "BOTTOMRIGHT")
    CUI.CoreFrame.Header.AddonLogo.highlight:SetTexture("Interface/Addons/weizPVP/Media/weizpvp_highlight.tga")
    CUI.CoreFrame.Header.AddonLogo.highlight:SetVertexColor(
        NS.ColorsLUT["InCombat"].rgb.r,
        NS.ColorsLUT["InCombat"].rgb.g,
        NS.ColorsLUT["InCombat"].rgb.b
    )
    CUI.CoreFrame.Header.AddonLogo.highlight.Pulse =
        CUI.CoreFrame.Header.AddonLogo.highlight.Pulse or CUI.CoreFrame.Header.AddonLogo.highlight:CreateAnimationGroup()
    CUI.CoreFrame.Header.AddonLogo.FadeIn =
        CUI.CoreFrame.Header.AddonLogo.FadeIn or CUI.CoreFrame.Header.AddonLogo.highlight.Pulse:CreateAnimation("Alpha")
    CUI.CoreFrame.Header.AddonLogo.FadeIn:SetDuration(0.15)
    CUI.CoreFrame.Header.AddonLogo.FadeIn:SetFromAlpha(0.2)
    CUI.CoreFrame.Header.AddonLogo.FadeIn:SetToAlpha(1)
    CUI.CoreFrame.Header.AddonLogo.FadeIn:SetEndDelay(0.55)
    CUI.CoreFrame.Header.AddonLogo.FadeIn:SetSmoothing("OUT")
    CUI.CoreFrame.Header.AddonLogo.FadeIn:SetOrder(1)
    CUI.CoreFrame.Header.AddonLogo.FadeOut =
        CUI.CoreFrame.Header.AddonLogo.FadeOut or CUI.CoreFrame.Header.AddonLogo.highlight.Pulse:CreateAnimation("Alpha")
    CUI.CoreFrame.Header.AddonLogo.FadeOut:SetDuration(0.3)
    CUI.CoreFrame.Header.AddonLogo.FadeOut:SetFromAlpha(1)
    CUI.CoreFrame.Header.AddonLogo.FadeOut:SetToAlpha(0.2)
    CUI.CoreFrame.Header.AddonLogo.FadeOut:SetOrder(2)
    CUI.CoreFrame.Header.AddonLogo.FadeOut:SetSmoothing("IN")
    CUI.CoreFrame.Header.AddonLogo.highlight.Pulse:SetLooping("REPEAT")
    CUI.CoreFrame.Header.AddonLogo.highlight.Pulse:SetScript(
        "OnStop",
        function()
            CUI.CoreFrame.Header.AddonLogo.highlight:SetAlpha(0)
        end
    )
    CUI.CoreFrame.Header.AddonLogo.highlight.Pulse:Play()
    CUI.CoreFrame.Header.AddonLogo.highlight.Pulse:Stop()

    --> Icon Overlay
    CUI.CoreFrame.Header.IconOverlay = CUI.CoreFrame.Header.IconOverlay or CreateFrame("Frame", nil, CUI.CoreFrame.Header)
    CUI.CoreFrame.Header.IconOverlay:SetPoint("TOPLEFT", CUI.CoreFrame.Header.AddonLogo, "TOPLEFT")
    CUI.CoreFrame.Header.IconOverlay:SetPoint("BOTTOMRIGHT", CUI.CoreFrame.Header.AddonLogo, "BOTTOMRIGHT")
    CUI.CoreFrame.Header.IconOverlay:SetAlpha(0)
    --:bg
    CUI.CoreFrame.Header.IconOverlay.bg = CUI.CoreFrame.Header.IconOverlay.bg or CUI.CoreFrame.Header.IconOverlay:CreateTexture(nil, "OVERLAY")
    CUI.CoreFrame.Header.IconOverlay.bg:SetPoint("CENTER", CUI.CoreFrame.Header.AddonLogo, "CENTER")
    CUI.CoreFrame.Header.IconOverlay.bg:SetHeight(NS.Options.Frames.Header.Height - Round(NS.Options.Frames.Header.Height / 3))
    CUI.CoreFrame.Header.IconOverlay.bg:SetWidth(NS.Options.Frames.Header.Height - Round(NS.Options.Frames.Header.Height / 3))
    CUI.CoreFrame.Header.IconOverlay.bg:SetTexture("Interface/Addons/weizPVP/Media/Icons/lock.tga")

    --> Title
    CUI.CoreFrame.Header.Title = CUI.CoreFrame.Header.Title or CUI.CoreFrame.Header:CreateFontString(nil, "ARTWORK", nil)
    CUI.CoreFrame.Header.Title:SetPoint("LEFT", CUI.CoreFrame.Header.AddonLogo, "RIGHT")
    CUI.CoreFrame.Header.Title:SetFont(
        SM:Fetch("font", NS.Options.Frames.Header.Font),
        NS.Options.Frames.Header.Height - Round(NS.Options.Frames.Header.Height / 4),
        "OUTLINE"
    )
    CUI.CoreFrame.Header.Title:SetText("|cff888888:|r")

    --> TitleVar
    CUI.CoreFrame.Header.TitleVar = CUI.CoreFrame.Header.TitleVar or CUI.CoreFrame.Header:CreateFontString(nil, "ARTWORK", nil)
    CUI.CoreFrame.Header.TitleVar:SetPoint("LEFT", CUI.CoreFrame.Header.Title, "RIGHT")
    CUI.CoreFrame.Header.TitleVar:SetFont(
        SM:Fetch("font", NS.Options.Frames.Header.Font),
        NS.Options.Frames.Header.Height - Round(NS.Options.Frames.Header.Height / 4),
        "OUTLINE"
    )
    CUI.CoreFrame.Header.TitleVar:SetText("|cFF26FF540|r")
    CUI.CoreFrame.Header.TitleVar:SetJustifyH("LEFT")

    --> Set Icon
    function CUI.CoreFrame.SetIcon(icon)
        if not icon then
            return
        end
        CUI.CoreFrame.Header.IconOverlay.bg:SetTexture("Interface/Addons/weizPVP/Media/Icons/" .. icon .. ".tga")
        if NS.ColorsLUT[icon] then
            CUI.CoreFrame.Header.IconOverlay.bg:SetVertexColor(
                Clamp(NS.ColorsLUT[icon].rgb.r, 0.7, 1),
                Clamp(NS.ColorsLUT[icon].rgb.g, 0.7, 1),
                Clamp(NS.ColorsLUT[icon].rgb.b, 0.7, 1)
            )
        end
        CUI.CoreFrame.Header.IconOverlay:SetAlpha(1)
        CUI.CoreFrame.Header.AddonLogo:Hide()
    end

    --> Toggle Pin
    function CUI.CoreFrame.TogglePin()
        if NS.Options.Window.Locked then
            NS.SetStatusMessage("Window is " .. sc.locked, true, 3)
            CUI.CoreFrame.SetIcon("lock")
            return
        end
        if NS.Options.Window.Pinned then
            NS.SetStatusMessage("Window is now " .. sc.unpinned, true, 3)
            CUI.CoreFrame.SetIcon("unpin")
            NS.Options.Window.Pinned = false
            CUI.CoreFrame:SetMovable(true)
        else
            NS.SetStatusMessage("Window is now " .. sc.pinned, true, 3)
            CUI.CoreFrame.SetIcon("pin")
            NS.Options.Window.Pinned = true
            CUI.CoreFrame:SetMovable(false)
        end
    end

    --> Toggle Lock
    function CUI.CoreFrame.ToggleLock()
        if NS.Options.Window.Locked then
            NS.SetStatusMessage("Window is now " .. sc.unlocked, true, 3)
            CUI.CoreFrame.SetIcon("unlock")
            NS.Options.Window.Locked = false
            CUI.CoreFrame:SetMovable(true)
        else
            NS.SetStatusMessage("Window is now " .. sc.locked, true, 3)
            CUI.CoreFrame.SetIcon("lock")
            NS.Options.Window.Locked = true
            NS.Options.Window.Pinned = false
            CUI.CoreFrame:SetMovable(false)
        end
    end
    --> Show Header Tooltip
    local function showHeaderToolTip()
        GameTooltip:ClearLines()
        GameTooltip:SetOwner(CUI.CoreFrame.Header, "ANCHOR_TOPLEFT")
        GameTooltip:AddLine("|TInterface\\Addons\\weizPVP\\Media\\weizpvp_nobg.tga:0|t " .. NS.Constants.AddonString)
        if NS.Options.Window.Locked == true then -- locked
            GameTooltip:AddLine("|cff42dcf4Ctrl + Right-Click|r to Unlock")
        elseif NS.Options.Window.Pinned == true then -- pinned
            GameTooltip:AddLine("|cff42dcf4Right-Click|r to unpin")
            GameTooltip:AddLine("|cff42dcf4Ctrl + Right-Click|r to Lock")
        else
            GameTooltip:AddLine("|cff42dcf4Right-Click|r to Pin")
            GameTooltip:AddLine("|cff42dcf4Ctrl + Right-Click|r to Lock")
        end
        GameTooltip:Show()
    end

    --> Scripts
    CUI.CoreFrame.Header:SetScript(
        "OnDragStart",
        function()
            if (not NS.Options.Window.Pinned) and (not NS.Options.Window.Locked) then
                GameTooltip:Hide()
                CUI.CoreFrame:SetMovable(true)
                CUI.CoreFrame:StartMoving()
                CUI.CoreFrame.moving = true
            elseif NS.Options.Window.Locked then
                NS.SetStatusMessage("Window is " .. sc.locked, true, 3)
                CUI.CoreFrame.SetIcon("lock")
            else
                NS.SetStatusMessage("Window is " .. sc.pinned, true, 3)
                CUI.CoreFrame.SetIcon("pin")
            end
        end
    )

    CUI.CoreFrame.Header:SetScript(
        "OnDragStop",
        function()
            GameTooltip:Hide()
            if CUI.CoreFrame.moving then
                CUI.CoreFrame:StopMovingOrSizing()
                CUI.CoreFrame.moving = nil
                NS.SaveCoreFramePosition()
            end
        end
    )

    CUI.CoreFrame.Header:SetScript(
        "OnMouseDown",
        function(_, button)
            GameTooltip:Hide()
            if button == "RightButton" then
                if IsControlKeyDown() then
                    CUI.CoreFrame.ToggleLock()
                else
                    CUI.CoreFrame.TogglePin()
                end
            end
        end
    )

    CUI.CoreFrame.Header:SetScript(
        "OnMouseUp",
        function(_, button)
            if button == "LeftButton" then
                CUI.CoreFrame:StopMovingOrSizing()
                CUI.CoreFrame.moving = nil
                CUI.CoreFrame.resizing = nil
                NS.SaveCoreFramePosition()
            end
            -- showHeaderToolTip()
        end
    )

    CUI.CoreFrame.Header:SetScript(
        "OnEnter",
        function()
            showHeaderToolTip()
        end
    )
    CUI.CoreFrame.Header:SetScript(
        "OnLeave",
        function()
            GameTooltip:Hide()
            GameTooltip:ClearLines()
        end
    )
end

--> Build Status Bar
local inDuration = 0.2
local outDuration = 0.3
local function BuildStatusBar()
    --> Status Bar
    CUI.CoreFrame.StatusBar = CUI.CoreFrame.StatusBar or CreateFrame("Frame", nil, CUI.CoreFrame)
    CUI.CoreFrame.StatusBar:SetFrameStrata("MEDIUM")
    CUI.CoreFrame.StatusBar:SetFrameLevel(0)
    CUI.CoreFrame.StatusBar:SetPoint("BOTTOMLEFT", CUI.CoreFrame.Header, "TOPLEFT", 0, -1)
    CUI.CoreFrame.StatusBar:SetPoint("BOTTOMRIGHT", CUI.CoreFrame.Header, "TOPRIGHT", 0, -1)
    CUI.CoreFrame.StatusBar:SetClipsChildren(true)
    CUI.CoreFrame.StatusBar:SetHeight(NS.Options.Frames.StatusPopUp.Height + 2)

    --> Status Bar - Clipped
    CUI.CoreFrame.StatusBar.Clipped = CUI.CoreFrame.StatusBar.Clipped or CreateFrame("Frame", nil, CUI.CoreFrame.StatusBar)
    CUI.CoreFrame.StatusBar.Clipped:SetPoint("TOPLEFT", CUI.CoreFrame.StatusBar, "BOTTOMLEFT", 0, NS.Options.Frames.BorderSize)
    CUI.CoreFrame.StatusBar.Clipped:SetPoint("TOPRIGHT", CUI.CoreFrame.StatusBar, "BOTTOMRIGHT", 0, -NS.Options.Frames.BorderSize)
    CUI.CoreFrame.StatusBar.Clipped:SetHeight(NS.Options.Frames.StatusPopUp.Height + NS.Options.Frames.BorderSize)
    CUI.CoreFrame.StatusBar.Clipped:SetBackdrop(defaultBackdrop)
    CUI.CoreFrame.StatusBar.Clipped:SetBackdropColor(unpack(NS.Options.Frames.StatusPopUp.BackgroundColor))
    CUI.CoreFrame.StatusBar.Clipped:SetBackdropBorderColor(unpack(NS.Options.Frames.StatusPopUp.BorderColor))

    --> Text
    CUI.CoreFrame.StatusBar.Clipped.Text =
        CUI.CoreFrame.StatusBar.Clipped.Text or CUI.CoreFrame.StatusBar.Clipped:CreateFontString(nil, "ARTWORK", nil)
    CUI.CoreFrame.StatusBar.Clipped.Text:SetPoint("TOPLEFT", CUI.CoreFrame.StatusBar.Clipped, "TOPLEFT")
    CUI.CoreFrame.StatusBar.Clipped.Text:SetPoint("BOTTOMRIGHT", CUI.CoreFrame.StatusBar.Clipped, "BOTTOMRIGHT")
    CUI.CoreFrame.StatusBar.Clipped.Text:SetFont(
        SM:Fetch("font", NS.Options.Frames.StatusPopUp.Font),
        Round(NS.Options.Frames.StatusPopUp.Height - (NS.Options.Frames.StatusPopUp.Height / 4)),
        NS.Options.Frames.StatusPopUp.FontOutline
    )
    CUI.CoreFrame.StatusBar.Clipped.Text:SetText("")
    CUI.CoreFrame.StatusBar.Clipped.Text:SetJustifyH("CENTER")
    CUI.CoreFrame.StatusBar.Clipped.Text:SetShadowColor(0, 0, 0, 0.6)
    CUI.CoreFrame.StatusBar.Clipped.Text:SetShadowOffset(1, -1)

    --> Animations
    --: Flash Status
    CUI.CoreFrame.StatusBar.FlashStatus = CUI.CoreFrame.StatusBar.FlashStatus or CUI.CoreFrame.StatusBar:CreateAnimationGroup()
    CUI.CoreFrame.StatusBar.Clipped.FadeIn = CUI.CoreFrame.StatusBar.Clipped.FadeIn or CUI.CoreFrame.StatusBar.FlashStatus:CreateAnimation("Alpha")
    CUI.CoreFrame.StatusBar.Clipped.FadeIn:SetDuration(inDuration)
    CUI.CoreFrame.StatusBar.Clipped.FadeIn:SetFromAlpha(0)
    CUI.CoreFrame.StatusBar.Clipped.FadeIn:SetToAlpha(1)
    CUI.CoreFrame.StatusBar.Clipped.FadeIn:SetEndDelay(2.5) -- Set the number of seconds the animation delays after finishing.
    CUI.CoreFrame.StatusBar.Clipped.FadeIn:SetOrder(1)
    CUI.CoreFrame.StatusBar.Clipped.SlideIn =
        CUI.CoreFrame.StatusBar.Clipped.SlideIn or CUI.CoreFrame.StatusBar.FlashStatus:CreateAnimation("Translation")
    CUI.CoreFrame.StatusBar.Clipped.SlideIn:SetDuration(inDuration)
    CUI.CoreFrame.StatusBar.Clipped.SlideIn:SetOffset(0, NS.Options.Frames.StatusPopUp.Height + NS.Options.Frames.BorderSize)
    CUI.CoreFrame.StatusBar.Clipped.SlideIn:SetEndDelay(2.5) -- Set the number of seconds the animation delays after finishing.
    CUI.CoreFrame.StatusBar.Clipped.SlideIn:SetOrder(1)
    CUI.CoreFrame.StatusBar.FlashStatus:SetScript(
        "OnPlay",
        function()
            if CUI.CoreFrame.Header.IconOverlay:GetAlpha() == 1 then
                CUI.CoreFrame.Header.AddonLogo:Hide()
            end
        end
    )
    CUI.CoreFrame.StatusBar.FlashStatus:SetScript(
        "OnFinished",
        function()
            CUI.CoreFrame.Header.AddonLogo:Show()
            CUI.CoreFrame.Header.IconOverlay:SetAlpha(0)
        end
    )
    CUI.CoreFrame.StatusBar.FlashStatus:SetScript(
        "OnStop",
        function()
            CUI.CoreFrame.Header.AddonLogo:Show()
            CUI.CoreFrame.Header.IconOverlay:SetAlpha(0)
        end
    )

    --> Slide Out
    CUI.CoreFrame.StatusBar.Clipped.FadeOut = CUI.CoreFrame.StatusBar.Clipped.FadeOut or CUI.CoreFrame.StatusBar.FlashStatus:CreateAnimation("Alpha")
    CUI.CoreFrame.StatusBar.Clipped.FadeOut:SetDuration(outDuration)
    CUI.CoreFrame.StatusBar.Clipped.FadeOut:SetFromAlpha(1)
    CUI.CoreFrame.StatusBar.Clipped.FadeOut:SetToAlpha(0)
    CUI.CoreFrame.StatusBar.Clipped.FadeOut:SetOrder(2)
    CUI.CoreFrame.StatusBar.Clipped.SlideOut =
        CUI.CoreFrame.StatusBar.Clipped.SlideOut or CUI.CoreFrame.StatusBar.FlashStatus:CreateAnimation("Translation")
    CUI.CoreFrame.StatusBar.Clipped.SlideOut:SetDuration(outDuration)
    CUI.CoreFrame.StatusBar.Clipped.SlideOut:SetOffset(0, -1 * select(2, CUI.CoreFrame.StatusBar.Clipped.SlideOut:GetOffset()))
    CUI.CoreFrame.StatusBar.Clipped.SlideOut:SetOrder(2)
end

--> Min/Max Button
local function BuildMinMaxButton()
    CUI.CoreFrame.Header.MinMaxButton = CUI.CoreFrame.Header.MinMaxButton or CreateFrame("Button", nil, CUI.CoreFrame.Header)
    CUI.CoreFrame.Header.MinMaxButton:SetPoint("RIGHT", CUI.CoreFrame.Header, "RIGHT", -Round(NS.Options.Frames.Header.Height / 6), 0)
    CUI.CoreFrame.Header.MinMaxButton:SetHeight(NS.Options.Frames.Header.Height - Round(NS.Options.Frames.Header.Height / 3))
    CUI.CoreFrame.Header.MinMaxButton:SetWidth(NS.Options.Frames.Header.Height - Round(NS.Options.Frames.Header.Height / 3))
    CUI.CoreFrame.Header.MinMaxButton:SetNormalTexture("Interface\\Addons\\weizPVP\\Media\\Textures\\arrow.tga")
    CUI.CoreFrame.Header.MinMaxButton:GetNormalTexture():SetVertexColor(0.8, 0.8, 0.8, 1)
    --: OnMouseDown
    CUI.CoreFrame.Header.MinMaxButton:SetScript(
        "OnMouseDown",
        function(_, button)
            if not InCombatLockdown() then
                if button == "LeftButton" then
                    if NS.Options.Window.Collapsed then
                        CUI.ExpandFrame()
                        NS.Options.Window.Collapsed = false
                    else
                        CUI.CollapseFrame()
                        NS.Options.Window.Collapsed = true
                    end
                    GameTooltip:ClearLines()
                    GameTooltip:SetOwner(CUI.CoreFrame.Header, "ANCHOR_TOPRIGHT")
                    if NS.Options.Window.Collapsed then
                        GameTooltip:SetText("Expand Window")
                    else
                        GameTooltip:SetText("Collapse Window")
                    end
                    GameTooltip:Show()
                end
            end
        end
    )
    --: OnEnter
    CUI.CoreFrame.Header.MinMaxButton:SetScript(
        "OnEnter",
        function(self)
            self:GetNormalTexture():SetVertexColor(1, 1, 1, 1)
            GameTooltip:ClearLines()
            GameTooltip:SetOwner(CUI.CoreFrame.Header, "ANCHOR_TOPRIGHT")
            if NS.Options.Window.Collapsed then
                GameTooltip:SetText("Expand Window")
            else
                GameTooltip:SetText("Collapse Window")
            end
            GameTooltip:Show()
        end
    )
    --: OnLeave
    CUI.CoreFrame.Header.MinMaxButton:SetScript(
        "OnLeave",
        function(self)
            self:GetNormalTexture():SetVertexColor(0.8, 0.8, 0.8, 1)
            GameTooltip:Hide()
        end
    )
end

--> Refresh Button
local function BuildRefreshButton()
    CUI.CoreFrame.Header.RefreshButton = CUI.CoreFrame.Header.RefreshButton or CreateFrame("Button", nil, CUI.CoreFrame)
    CUI.CoreFrame.Header.RefreshButton:SetPoint("RIGHT", CUI.CoreFrame.Header.MinMaxButton, "LEFT", -Round(NS.Options.Frames.Header.Height / 6), 0)
    CUI.CoreFrame.Header.RefreshButton:SetHeight(NS.Options.Frames.Header.Height - Round(NS.Options.Frames.Header.Height / 3))
    CUI.CoreFrame.Header.RefreshButton:SetWidth(NS.Options.Frames.Header.Height - Round(NS.Options.Frames.Header.Height / 3))
    CUI.CoreFrame.Header.RefreshButton:SetNormalTexture("Interface\\Addons\\weizPVP\\Media\\Textures\\button_refresh.tga")
    CUI.CoreFrame.Header.RefreshButton:GetNormalTexture():SetVertexColor(0.9, 0.6, 0)
    --> Animations
    CUI.CoreFrame.Header.RefreshButton.AniGroup =
        CUI.CoreFrame.Header.RefreshButton.AniGroup or CUI.CoreFrame.Header.RefreshButton:CreateAnimationGroup()
    CUI.CoreFrame.Header.RefreshButton.Rotate =
        CUI.CoreFrame.Header.RefreshButton.Rotate or CUI.CoreFrame.Header.RefreshButton.AniGroup:CreateAnimation("Rotation")
    CUI.CoreFrame.Header.RefreshButton.Rotate:SetDegrees(360)
    CUI.CoreFrame.Header.RefreshButton.Rotate:SetDuration(0.4)
    CUI.CoreFrame.Header.RefreshButton:SetScript(
        "OnMouseDown",
        function(self, button)
            if button == "LeftButton" then
                NS.ClearListData()
                self.AniGroup:Play()
            end
        end
    )
    CUI.CoreFrame.Header.RefreshButton:SetScript(
        "OnEnter",
        function(self, _)
            self:GetNormalTexture():SetVertexColor(1, 0.7, 0)
            GameTooltip:ClearLines()
            GameTooltip:SetOwner(CUI.CoreFrame.Header, "ANCHOR_TOPRIGHT")
            GameTooltip:SetText("Clear List")
            GameTooltip:Show()
        end
    )
    CUI.CoreFrame.Header.RefreshButton:SetScript(
        "OnLeave",
        function(self)
            self:GetNormalTexture():SetVertexColor(0.9, 0.6, 0)
            GameTooltip:Hide()
        end
    )
end

--> Scroll Frame
local function BuildScrollFrame()
    CUI.CoreFrame.ScrollFrame = CUI.CoreFrame.ScrollFrame or CreateFrame("ScrollFrame", nil, CUI.CoreFrame)
    CUI.CoreFrame.ScrollFrame:EnableMouseWheel(true)
    CUI.CoreFrame.ScrollFrame:SetClipsChildren(true)
    CUI.CoreFrame.ScrollFrame:SetPoint("TOPLEFT", CUI.CoreFrame, "TOPLEFT")
    CUI.CoreFrame.ScrollFrame:SetPoint("BOTTOMRIGHT", CUI.CoreFrame, "BOTTOMRIGHT")
    CUI.CoreFrame.ScrollFrame:SetBackdrop(defaultBackdrop)
    CUI.CoreFrame.ScrollFrame:SetBackdropColor(unpack(NS.Options.Frames.BackgroundColor))
    CUI.CoreFrame.ScrollFrame:SetBackdropBorderColor(0, 0, 0, 1)
    --: ScrollFrame: MOUSEWHEEL
    CUI.CoreFrame.ScrollFrame:SetScript(
        "OnMouseWheel",
        function(_, value)
            CUI.AdustScrollOffset(value)
        end
    )
    --: ListFrame - ScrollFrame scroll child
    CUI.CoreFrame.ScrollFrame.ListFrame = CUI.CoreFrame.ScrollFrame.ListFrame or CreateFrame("Frame", nil, CUI.CoreFrame.ScrollFrame)
    CUI.CoreFrame.ScrollFrame.ListFrame:SetPoint("CENTER", CUI.CoreFrame.ScrollFrame, "CENTER")
    CUI.CoreFrame.ScrollFrame.ListFrame:SetPoint("TOP", CUI.CoreFrame, "BOTTOM")
    CUI.CoreFrame.ScrollFrame.ListFrame:SetHeight(NS.Options.Frames.List.Height)
    CUI.CoreFrame.ScrollFrame.ListFrame:SetWidth(CUI.CoreFrame.ScrollFrame:GetWidth())

    --: Set ListFrame as scroll child to ScrollFrame
    CUI.CoreFrame.ScrollFrame:SetScrollChild(CUI.CoreFrame.ScrollFrame.ListFrame)
end

--> Targeted Icon
local function BuildTargetedIcon()
    --: Outer Icon Frame
    CUI.CoreFrame.OuterIconFrame = CUI.CoreFrame.OuterIconFrame or CreateFrame("Frame", nil, CUI.CoreFrame)
    CUI.CoreFrame.OuterIconFrame:SetPoint("TOPLEFT", CUI.CoreFrame, "TOPLEFT", -NS.Options.Bars.RowHeight, 0)
    CUI.CoreFrame.OuterIconFrame:SetPoint("BOTTOMRIGHT", CUI.CoreFrame, "BOTTOMRIGHT", NS.Options.Bars.RowHeight, 0)
    CUI.CoreFrame.OuterIconFrame:SetClipsChildren(true)
    --: Targeted Icon
    CUI.CoreFrame.TargetedIcon = CUI.CoreFrame.TargetedIcon or CUI.CoreFrame.OuterIconFrame:CreateTexture(nil, "OVERLAY")
    CUI.CoreFrame.TargetedIcon:SetTexture("Interface\\Addons\\weizPVP\\Media\\Textures\\target.tga") --32x32 image
    CUI.CoreFrame.TargetedIcon:SetSize(
        Round(NS.Options.Bars.RowHeight - (NS.Options.Bars.RowHeight / 6)),
        Round(NS.Options.Bars.RowHeight - (NS.Options.Bars.RowHeight / 6))
    )
    CUI.CoreFrame.TargetedIcon:Hide()
end

--> Resize Buttons
local function BuildResizeButtons()
    --> BOTTOM RIGHT
    CUI.CoreFrame.DragBottomRight = CUI.CoreFrame.DragBottomRight or CreateFrame("Button", "DragBottomRight", CUI.CoreFrame)
    CUI.CoreFrame.DragBottomRight:SetNormalTexture("Interface\\Addons\\weizPVP\\Media\\Textures\\resizeGrip.tga")
    CUI.CoreFrame.DragBottomRight:SetHighlightTexture("Interface\\Addons\\weizPVP\\Media\\Textures\\resizeGrip.tga")
    CUI.CoreFrame.DragBottomRight:GetNormalTexture():SetVertexColor(1, 1, 1, 0.3)
    CUI.CoreFrame.DragBottomRight:GetHighlightTexture():SetVertexColor(1, 0.8, 0, 1)
    CUI.CoreFrame.DragBottomRight:SetSize(16, 16)
    CUI.CoreFrame.DragBottomRight:SetFrameLevel(10)
    CUI.CoreFrame.DragBottomRight:SetPoint("BOTTOMRIGHT", CUI.CoreFrame.ScrollFrame, "BOTTOMRIGHT", 0, 0)
    --: OnMouseDown
    CUI.CoreFrame.DragBottomRight:SetScript(
        "OnMouseDown",
        function(_, button)
            if (not NS.Options.Window.Locked) and button == "LeftButton" then
                CUI.CoreFrame.resizing = true
                CUI.CoreFrame:StopMovingOrSizing("TOPLEFT")
                CUI.CoreFrame:StartSizing("BOTTOMRIGHT")
            elseif NS.Options.Window.Locked == true then
                NS.SetStatusMessage("Window is " .. sc.locked, true, 3)
                CUI.CoreFrame.SetIcon("lock")
            end
        end
    )
    --: OnMouseUp
    CUI.CoreFrame.DragBottomRight:SetScript(
        "OnMouseUp",
        function(_, button)
            if (button == "LeftButton") then
                if not NS.Options.Window.Locked then
                    CUI.SnapHeightToBars()
                    if CUI.CoreFrame.resizing == true then
                        CUI.CoreFrame:StopMovingOrSizing()
                        CUI.CoreFrame.resizing = nil
                    end
                    NS.RefreshCurrentList()
                end
            end
        end
    )
    --> BOTTOM LEFT
    CUI.CoreFrame.DragBottomLeft = CUI.CoreFrame.DragBottomLeft or CreateFrame("Button", "DragBottomLeft", CUI.CoreFrame)
    CUI.CoreFrame.DragBottomLeft:SetNormalTexture("Interface\\Addons\\weizPVP\\Media\\Textures\\resizeGrip.tga")
    CUI.CoreFrame.DragBottomLeft:SetHighlightTexture("Interface\\Addons\\weizPVP\\Media\\Textures\\resizeGrip.tga")
    CUI.CoreFrame.DragBottomLeft:GetNormalTexture():SetVertexColor(1, 1, 1, 0.3)
    CUI.CoreFrame.DragBottomLeft:GetHighlightTexture():SetVertexColor(1, 0.8, 0, 1)
    CUI.CoreFrame.DragBottomLeft:GetNormalTexture():SetTexCoord(1, 0, 0, 1)
    CUI.CoreFrame.DragBottomLeft:GetHighlightTexture():SetTexCoord(1, 0, 0, 1)
    CUI.CoreFrame.DragBottomLeft:SetSize(16, 16)
    CUI.CoreFrame.DragBottomLeft:SetFrameLevel(10)
    CUI.CoreFrame.DragBottomLeft:SetPoint("BOTTOMLEFT", CUI.CoreFrame.ScrollFrame, "BOTTOMLEFT", 0, 0)
    --: OnMouseDown
    CUI.CoreFrame.DragBottomLeft:SetScript(
        "OnMouseDown",
        function(_, button)
            if (not NS.Options.Window.Locked) and button == "LeftButton" then
                CUI.CoreFrame.resizing = true
                CUI.CoreFrame:StopMovingOrSizing("TOPRIGHT")
                CUI.CoreFrame:StartSizing("BOTTOMLEFT")
            elseif NS.Options.Window.Locked == true then
                NS.SetStatusMessage("Window is " .. sc.locked, true, 3)
                CUI.CoreFrame.SetIcon("lock")
            end
        end
    )
    --: OnMouseUp
    CUI.CoreFrame.DragBottomLeft:SetScript(
        "OnMouseUp",
        function(_, button)
            if (button == "LeftButton") then
                if not NS.Options.Window.Locked then
                    CUI.SnapHeightToBars()
                    if CUI.CoreFrame.resizing == true then
                        CUI.CoreFrame:StopMovingOrSizing()
                        CUI.CoreFrame.resizing = nil
                    end
                    NS.RefreshCurrentList()
                end
            end
        end
    )
end

--|> Initialize
-----------------------------------------------------------
local initializing = false
function CUI.Initialize()
    if initializing == true then
        return
    end
    if CUI.CoreFrame and CUI.CoreFrame.StatusBar then
        CUI.CoreFrame:StopAnimating()
    else
        localInit()
    end
    initializing = true
    BuildCoreFrame()
    BuildHeaderFrame()
    BuildStatusBar()
    BuildMinMaxButton()
    BuildRefreshButton()
    BuildScrollFrame()
    BuildTargetedIcon()
    BuildResizeButtons()
    CUI.BuildBars()
    NS.SetCoreFramePosition()
    NS.SetWindowSettings()
    NS.ManageBarsDisplayed()
    NS.RefreshCurrentList()
    NS.UpdateNearbyCount()
    CUI.SnapHeightToBars()
    CUI.ChangeTargetIcon()
    initializing = false
end

--|> Functions
-----------------------------------------------------------
--> Collapse Frame
function CUI.CollapseFrame()
    CUI.CoreFrame.Header.MinMaxButton:GetNormalTexture():SetRotation(3.14159)
    UIFrameFadeIn(CUI.CoreFrame.Header.MinMaxButton, 0.2, 0, 1)
    CUI.CoreFrame.ScrollFrame:Hide()
    CUI.CoreFrame.TargetedIcon:Hide()
    CUI.CoreFrame:SetClampedToScreen(false)
    CUI.CoreFrame.DragBottomLeft:Hide()
    CUI.CoreFrame.DragBottomRight:Hide()
    NS.Options.Window.Collapsed = true
end

--> Expand Frame
function CUI.ExpandFrame()
    CUI.CoreFrame.Header.MinMaxButton:GetNormalTexture():SetRotation(0)
    UIFrameFadeIn(CUI.CoreFrame.Header.MinMaxButton, 0.2, 0, 1)
    CUI.CoreFrame.ScrollFrame:Show()
    CUI.CoreFrame.TargetedIcon:Show()
    CUI.CoreFrame:SetClampedToScreen(true)
    CUI.CoreFrame.DragBottomLeft:Show()
    CUI.CoreFrame.DragBottomRight:Show()
    NS.Options.Window.Collapsed = false
end

--> Adjust Scroll Offset
function CUI.AdustScrollOffset(value)
    if not InCombatLockdown() then
        local vertScroll = CUI.CoreFrame.ScrollFrame:GetVerticalScroll()
        local rowHeight = NS.Options.Bars.RowHeight + NS.Options.Bars.VerticalSpacing
        local offsetMax = CUI.CoreFrame.ScrollFrame.ListFrame:GetHeight() - CUI.CoreFrame.ScrollFrame:GetHeight()
        local numPlayersOnBars = 0
        for _ in pairs(NS.PlayersOnBars) do
            numPlayersOnBars = numPlayersOnBars + 1
        end
        local barsInDisplay = floor(CUI.CoreFrame:GetHeight() / rowHeight)
        local rawCalc = CUI.CoreFrame:GetHeight() / rowHeight
        if rawCalc - barsInDisplay >= .5 then
            barsInDisplay = barsInDisplay + 1
        end
        if barsInDisplay < 1 then
            barsInDisplay = 1
        end
        local minVerticalScroll = (numPlayersOnBars - barsInDisplay) * rowHeight
        if minVerticalScroll < 0 then
            minVerticalScroll = 0
        end
        local offset
        if value > 0 then
            offset = vertScroll - (rowHeight)
        else
            offset = vertScroll + (rowHeight)
        end
        if offset < 0 then
            CUI.CoreFrame.ScrollFrame:SetVerticalScroll(0)
        elseif offset > offsetMax then
            CUI.CoreFrame.ScrollFrame:SetVerticalScroll(offsetMax)
        elseif minVerticalScroll < offset then
            CUI.CoreFrame.ScrollFrame:SetVerticalScroll(minVerticalScroll)
        else
            CUI.CoreFrame.ScrollFrame:SetVerticalScroll(offset)
        end
    end
end

--> Auto Vertical Scroll - for snapping height to bars
local function AutoVerticalScroll()
    if not InCombatLockdown() then
        local frameHeight = CUI.CoreFrame:GetHeight()
        local rowHeight = NS.Options.Bars.RowHeight + NS.Options.Bars.VerticalSpacing
        local rawCalc = frameHeight / rowHeight
        local barsInDisplay = floor(frameHeight / rowHeight)
        if rawCalc - barsInDisplay >= .5 then
            barsInDisplay = barsInDisplay + 1
        end
        if barsInDisplay < 1 then
            barsInDisplay = 1
        end
        local numPlayersOnBars = 0
        for _ in pairs(NS.PlayersOnBars) do
            numPlayersOnBars = numPlayersOnBars + 1
        end
        local minVerticalScroll = (numPlayersOnBars - barsInDisplay) * rowHeight
        if barsInDisplay > numPlayersOnBars then
            CUI.CoreFrame.ScrollFrame:SetVerticalScroll(0)
        elseif minVerticalScroll < CUI.CoreFrame.ScrollFrame:GetVerticalScroll() then
            CUI.CoreFrame.ScrollFrame:SetVerticalScroll(minVerticalScroll)
        end
    end
end

--> Snap to Bars
function CUI.SnapHeightToBars()
    if not InCombatLockdown() then
        local frameHeight = CUI.CoreFrame:GetHeight()
        local rowHeight = NS.Options.Bars.RowHeight + NS.Options.Bars.VerticalSpacing
        local rawCalc = frameHeight / rowHeight
        local barsInDisplay = floor(frameHeight / rowHeight)
        if rawCalc - barsInDisplay >= .5 then
            barsInDisplay = barsInDisplay + 1
        end
        if barsInDisplay < 1 then
            barsInDisplay = 1
        end
        CUI.CoreFrame:SetHeight(barsInDisplay * rowHeight)
        AutoVerticalScroll()
        NS.SaveCoreFramePosition()
    end
end

--> Change Target Icon
local lastRowSelected = nil
function CUI.ChangeTargetIcon()
    if not CUI.CoreFrame then
        return
    end
    local GUID = UnitGUID("target")
    if NS.PlayersOnBars[GUID] then
        if CUI.CoreFrame:GetLeft() < 8 then
            CUI.CoreFrame.TargetedIcon:SetRotation(3.14159)
            CUI.CoreFrame.TargetedIcon:ClearAllPoints()
            CUI.CoreFrame.TargetedIcon:SetPoint("LEFT", CUI.Bar[NS.PlayersOnBars[GUID]], "RIGHT", -(NS.Options.Frames.BorderSize), 0)
        else
            CUI.CoreFrame.TargetedIcon:SetRotation(0)
            CUI.CoreFrame.TargetedIcon:ClearAllPoints()
            CUI.CoreFrame.TargetedIcon:SetPoint("RIGHT", CUI.Bar[NS.PlayersOnBars[GUID]], "LEFT", NS.Options.Frames.BorderSize, 0)
        end
        CUI.CoreFrame.TargetedIcon:Show()
        if lastRowSelected then
            CUI.Bar[lastRowSelected]:SetStatusBarTexture(SM:Fetch("statusbar", NS.Options.Bars.Texture))
        end
        CUI.Bar[NS.PlayersOnBars[GUID]]:SetStatusBarTexture(SM:Fetch("statusbar", NS.Options.Bars.BarSolid))
        lastRowSelected = NS.PlayersOnBars[GUID] or nil
    else
        CUI.CoreFrame.TargetedIcon:Hide()
        if lastRowSelected then
            CUI.Bar[lastRowSelected]:SetStatusBarTexture(SM:Fetch("statusbar", NS.Options.Bars.Texture))
        end
    end
end

--> Window: Set Window Settings
function NS.SetWindowSettings()
    if not InCombatLockdown() then
        if NS.Options.Window.Visible == true then
            NS.CoreUI.CoreFrame:Show()
            if NS.Options.Window.Collapsed == false then
                NS.CoreUI.ExpandFrame()
            else
                NS.CoreUI.CollapseFrame()
            end
        else
            NS.CoreUI.CoreFrame:Hide()
        end
    end
end

--> Set Status Bar Message
function NS.SetStatusMessage(StatusBarText, QuickMessage, Duration)
    if QuickMessage == true then -- Message will be temporary
        CUI.CoreFrame.StatusBar.FlashStatus:Stop()
        CUI.CoreFrame.StatusBar.Clipped.Text:SetText(StatusBarText)
        CUI.CoreFrame.StatusBar.Clipped.SlideIn:SetEndDelay(Duration)
        CUI.CoreFrame.StatusBar.Clipped.FadeIn:SetEndDelay(Duration)
        CUI.CoreFrame.StatusBar.FlashStatus:Play()
    else -- Static Message
        CUI.CoreFrame.StatusBar.FlashStatus:Stop()
        CUI.CoreFrame.StatusBar.Clipped.Text:SetText(StatusBarText)
        CUI.CoreFrame.StatusBar.Clipped.SlideIn:SetEndDelay(1200)
        CUI.CoreFrame.StatusBar.Clipped.FadeIn:SetEndDelay(1200)
        CUI.CoreFrame.StatusBar.FlashStatus:Play()
    end
end
