-----------------------------------------------------------------------
--|> EVENTS
-----------------------------------------------------------------------
local _, NS = ...
local InCombatLockdown = InCombatLockdown
local wipe = wipe
local weizPVP = _G.weizPVP

NS.PlayerActiveCache = {}

--|> EVENT MONITORING
-----------------------------------------------------------------------
--> ENABLE EVENTS
function NS.EnableEvents()
    --> ADDON
    weizPVP:RegisterEvent("ADDON_LOADED", NS.AddonLoadedEvent)
    --> DATA COLLECTION
    weizPVP:RegisterEvent("NAME_PLATE_UNIT_ADDED", NS.NameplateAdded)
    weizPVP:RegisterEvent("NAME_PLATE_UNIT_REMOVED", NS.NameplateRemoved)
    weizPVP:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", NS.CombatLogEvent)
    weizPVP:RegisterEvent("PLAYER_TARGET_CHANGED", NS.PlayerTargetEvent)
    weizPVP:RegisterEvent("UPDATE_MOUSEOVER_UNIT", NS.PlayerMouseoverEvent)
    weizPVP:RegisterEvent("UNIT_HEALTH_FREQUENT", NS.UnitHealthEvent)
    weizPVP:RegisterEvent("UNIT_TARGET", NS.UnitTargetEvent)
    -- weizPVP:RegisterEvent("UNIT_DYNAMIC_FLAGS", NS.UnitFlagsChanged)
    weizPVP:RegisterEvent("UNIT_FLAGS", NS.UnitFlagsChanged)
    --> ZONE CHANGED
    weizPVP:RegisterEvent("PLAYER_ENTERING_BATTLEGROUND", NS.PlayerEnteringBattlegroundEvent)
    weizPVP:RegisterEvent("PLAYER_ENTERING_WORLD", NS.PlayerEnteringWorldEvent)
    weizPVP:RegisterEvent("ZONE_CHANGED_NEW_AREA", NS.ZoneChangedNewAreaEvent)
    --> COMBAT/PVP
    weizPVP:RegisterEvent("PLAYER_REGEN_DISABLED", NS.EnteringCombat)
    weizPVP:RegisterEvent("PLAYER_REGEN_ENABLED", NS.LeavingCombat)
    --> PLAYER UPDATES
    weizPVP:RegisterEvent("PLAYER_LEVEL_UP", NS.PlayerLevelUp)
    --> DISPLAY CHANGES
    weizPVP:RegisterEvent("DISPLAY_SIZE_CHANGED", NS.UpdateDisplayData)
    -- weizPVP:RegisterEvent("PLAYER_LOGIN", NS.UpdateDisplayData)
    weizPVP:RegisterEvent("LOADING_SCREEN_DISABLED", NS.LoadingScreenLeave)
    --> Crosshair
    NS.Crosshair.Enable()
end

--> DISABLE EVENTS
function NS.DisableEvents()
    --> ADDON
    weizPVP:UnregisterEvent("ADDON_LOADED", NS.AddonLoadedEvent)
    --> DATA COLLECTION
    weizPVP:UnregisterEvent("NAME_PLATE_UNIT_ADDED", NS.NameplateAdded)
    weizPVP:UnregisterEvent("NAME_PLATE_UNIT_REMOVED", NS.NameplateRemoved)
    weizPVP:UnregisterEvent("COMBAT_LOG_EVENT_UNFILTERED", NS.CombatLogEvent)
    weizPVP:UnregisterEvent("PLAYER_TARGET_CHANGED", NS.PlayerTargetEvent)
    weizPVP:UnregisterEvent("UPDATE_MOUSEOVER_UNIT", NS.PlayerMouseoverEvent)
    weizPVP:UnregisterEvent("UNIT_HEALTH_FREQUENT", NS.UnitHealthEvent)
    weizPVP:UnregisterEvent("UNIT_TARGET", NS.UnitTargetEvent)
    -- weizPVP:UnregisterEvent("UNIT_DYNAMIC_FLAGS", NS.UnitTargetEvent)
    weizPVP:UnregisterEvent("UNIT_FLAGS", NS.UnitTargetEvent)
    --> COMBAT/PVP
    weizPVP:UnregisterEvent("PLAYER_REGEN_DISABLED", NS.EnteringCombat)
    weizPVP:UnregisterEvent("PLAYER_REGEN_ENABLED", NS.LeavingCombat)
    --> PLAYER UPDATES
    weizPVP:UnregisterEvent("PLAYER_LEVEL_UP", NS.PlayerLevelUp)
    --> DISPLAY CHANGES
    weizPVP:UnregisterEvent("DISPLAY_SIZE_CHANGED", NS.UpdateDisplayData)
    -- weizPVP:UnregisterEvent("PLAYER_LOGIN", NS.UpdateDisplayData)
    weizPVP:UnregisterEvent("LOADING_SCREEN_DISABLED", NS.LoadingScreenLeave)
    -->Crosshair
    NS.Crosshair.Disable()
end

--|> UNIT EVENTS
-----------------------------------------------------------------------
--> EVENT: MOUSEOVER
function NS.PlayerMouseoverEvent()
    NS.GetUnitData("mouseover")
end

--> EVENT: TARGET CHANGED
function NS.PlayerTargetEvent()
    NS.GetUnitData("target")
    NS.Crosshair.NewTarget() -- Crosshair-New Target
    NS.CoreUI.ChangeTargetIcon()
end

--> EVENT: NAMEPLATE ADDED
function NS.NameplateAdded(_, unit)
    if NS.IsUnitValidForTracking(unit) then
        NS.Crosshair.NameplateAdded(unit)
        NS.GetUnitData(unit)
        NS.CurrentNameplates[UnitGUID(unit)] = true
    end
end

--> EVENT: NAMEPLATE REMOVED
function NS.NameplateRemoved(_, unit)
    if NS.IsUnitValidForTracking(unit) then
        NS.Crosshair.NameplateRemoved(unit)
        if NS.IsUnitValidForTracking(unit) then
            NS.CurrentNameplates[UnitGUID(unit)] = nil
        end
    end
end

--> EVENT: HEALTH
function NS.UnitHealthEvent(_, unit)
    if NS.IsUnitValidForTracking(unit) then
        if NS.PlayersOnBars[UnitGUID(unit)] then
            NS.UnitHealthCheck(unit)
        end
    end
end

--> EVENT: UNIT TARGET
function NS.UnitTargetEvent(_, unit)
    NS.GetUnitData(unit)
end

--> EVENT: UNIT FLAGS CHANGED
function NS.UnitFlagsChanged(_, unit)
    local GUID = UnitGUID(unit)
    if (not NS.IsUnitValidForTracking(unit)) and NS.PlayerActiveCache[GUID] then
        wipe(NS.PlayerActiveCache[GUID])
        wipe(NS.CurrentList[GUID])
        NS.SortNearbyList()
        if NS.PlayersOnBars[GUID] then
            NS.RefreshCurrentList()
        end
    end
end

--|> ADDON EVENTS
-----------------------------------------------------------------------
--> EVENT: ADDON_LOADED
function NS.AddonLoadedEvent()
    NS.Crosshair.OnLoad()
    weizPVP:UnregisterEvent("ADDON_LOADED", "AddonLoadedEvent") --disabled after initial load
end

--|> COMBAT EVENTS
-----------------------------------------------------------------------
--> EVENT: ENTERING COMBAT
function NS.EnteringCombat()
    NS.SetStatusMessage("|cFF" .. NS.ColorsLUT["InCombat"].hex .. "+++ IN COMBAT +++|r", true, 3)
    NS.CoreUI.CoreFrame.Header.AddonLogo.bg:SetTexture("Interface/Addons/weizPVP/Media/weizpvp_white.tga")
    NS.CoreUI.CoreFrame.Header.AddonLogo.highlight.Pulse:Play()
end

--> EVENT: LEAVING COMBAT
function NS.LeavingCombat()
    NS.SetStatusMessage("|cFF" .. NS.ColorsLUT["OutOfCombat"].hex .. "--- OUT OF COMBAT ---|r", true, 5)
    NS.ManageListTimeouts()
    NS.CoreUI.CoreFrame.Header.AddonLogo.highlight.Pulse:Stop()
    NS.CoreUI.CoreFrame.Header.AddonLogo.bg:SetTexture("Interface/Addons/weizPVP/Media/weizpvp_green.tga")
    C_Timer.After(
        2,
        function()
            if not InCombatLockdown() then
                NS.CoreUI.CoreFrame.Header.AddonLogo.bg:SetTexture("Interface/Addons/weizPVP/Media/weizpvp_nobg.tga")
            end
        end
    )
end
