-----------------------------------------------------------
--|> UNIT PROCESSING
--: Process Unit Data
-----------------------------------------------------------
local _, NS = ... --> Addon Name, Namespace
--> UpValues
local UnitName = UnitName
local GetGuildInfo = GetGuildInfo
local GetTime = GetTime
local string_find = string.find
local UnitCanAttack = UnitCanAttack
local UnitExists = UnitExists
local UnitClass = UnitClass
local UnitRace = UnitRace
local UnitIsDeadOrGhost = UnitIsDeadOrGhost
local UnitOnTaxi = UnitOnTaxi
local UnitIsPlayer = UnitIsPlayer
local GetUnitName = GetUnitName
local gsub = gsub

--> GET UNIT DATA
local unitUpdateThreshold = 10
-- Gets data for events with a UNIT ID; such as 'target' or 'nameplate'
function NS.GetUnitData(unit)
    if (not unit) or (not UnitExists(unit)) or (not NS.IsUnitValidForTracking(unit)) then
        return -- no valid unit to process
    end
    -- valid unit exists: get info!
    local name = NS.GetUnitName(unit)
    local GUID = UnitGUID(unit) or nil
    if name and GUID then
        --: Add player to DB if not found
        if not NS.PlayerDB[name] then
            NS.PlayerDB[name] = {}
            NS.PlayerDB[name].Name = name
            NS.PlayerDB[name].Timestamp = 0
            _, NS.PlayerDB[name].Class = UnitClass(unit)
        end

        local role
        --: Update player info if edstimated or past update threshold
        if NS.PlayerDB[name].Estimated == true or NS.PlayerDB[name].Timestamp + unitUpdateThreshold > GetTime() then
            NS.PlayerDB[name].Guild = GetGuildInfo(unit)
            NS.PlayerDB[name].Level = UnitLevel(unit)
            NS.PlayerDB[name].Race = UnitRace(unit)
            role = NS.ClassRoleAssign(UnitClass(unit))
        end
        --: Player On Bars?
        if NS.PlayersOnBars[GUID] then
            local OnTaxi = UnitOnTaxi(unit)
            if OnTaxi then
                NS.PlayerActiveCache[GUID].OnTaxi = true
            elseif not OnTaxi and NS.PlayerActiveCache[GUID].OnTaxi == true then
                NS.PlayerActiveCache[GUID].OnTaxi = false
            end
            NS.UnitHealthCheck(unit)
        end
        NS.PlayerDB[name].Estimated = false
        NS.PlayerDB[name].Timestamp = GetTime()
        NS.UpdatePlayerActiveCache(name, nil, nil, role, GUID) -- (name, stealth, dead, role, GUID)
        return
    end
end

--> Is Unit Valid For Tracking
function NS.IsUnitValidForTracking(unit)
    if (not unit) or (not UnitIsPlayer(unit)) then -- input check
        return false
    end
    if string_find(GetUnitName(unit), "Unknown") then
        return false
    end
    if (UnitCanAttack("player", unit) or UnitIsEnemy("player", unit) or NS.Options.DEBUG) then -- attackable player check
        return true -- VALID
    else
        local GUID = UnitGUID(unit) or nil
        if GUID and NS.PlayerActiveCache[GUID] then
            NS.PlayerActiveCache[GUID] = nil
            NS.PlayersOnBars[GUID] = nil
            NS.ActiveList[GUID] = nil
            NS.ActiveDeadList[GUID] = nil
            NS.InactiveList[GUID] = nil
            NS.InactiveDeadList[GUID] = nil
        end
    end
    return false
end

--> weizPVP: GetUnitName()
--: @returns "playerName-realmName" for all players, no matter the realm
function NS.GetUnitName(unit)
    if not unit then -- check for unit
        return
    end
    local name, realm = UnitName(unit, true) -- check for unit name
    if not name then
        return
    end
    if not realm then -- same realm
        return name .. "-" .. NS.PlayerRealm
    else -- different realm
        realm = gsub(realm, "[%s%-]", "")
        return name .. "-" .. realm
    end
end

--> UnitHealthCheck
function NS.UnitHealthCheck(unit)
    -- if (not unit) or (not UnitExists(unit)) or (not NS.IsUnitValidForTracking(unit)) then
    --     return -- no valid unit to process
    -- end
    local GUID = UnitGUID(unit) or nil
    if GUID then
        -- local dead =
        -- if dead ~= nil then
        NS.PlayerActiveCache[GUID].Dead = UnitIsDeadOrGhost(unit)
        -- end
        local currentHealth = (UnitHealth(unit) / UnitHealthMax(unit) * 100) or nil
        if currentHealth then
            NS.PlayerActiveCache[GUID].Health = currentHealth
        end
        -- if dead or currentHealth then
        NS.RefreshBarByGUID(GUID)
    -- end
    end
end
